*(1) Read in raw survey data and save as SPSS datafile

GET DATA  /TYPE = TXT
 /FILE = 'CO_Dummy_survey.txt'
 /DELCASE = LINE
 /DELIMITERS = " "
 /ARRANGEMENT = DELIMITED
 /FIRSTCASE = 1
 /IMPORTCASE = ALL
 /VARIABLES =
 Record F2.0
 HH_ID F2.0
 Person F1.0
 Age F2.0
 Sex F1.0
 Cars F1.0
 HH_Size F1.0
 .
CACHE.
EXECUTE.
DATASET NAME DataSet1 WINDOW=FRONT.

Variable label Record 'Unique record [person] identifier'.
Variable label HH_ID 'Unique household identifier'.
Variable label Person 'Person no. within household'.
Variable label Age 'Age (years)'.
Variable label Sex 'Sex'.
Variable label Cars 'Cars in household'.
Variable label HH_Size 'Persons in household'.

Value labels Sex 1 'Male' 2 'Female'.

SAVE OUTFILE='CO_Dummy_survey.sav'
 /COMPRESSED.


*(2) Recode survey variables to match categories/classes used in constraint tables

*(a) Age

RECODE  Age  (0 thru 15=1)  (16 thru 64=2)  (65 thru Highest=3)  INTO  Age_gp .
VARIABLE LABELS Age_gp 'Grouped Age'.
VALUE LABELS Age_gp 1 'Child' 2 'Adult' 3 'Pensioner'.
EXECUTE .

*(b) Sex [already in required format]
*(c) Cars [already in required format]
*(d) HH_Size [already in required format]

*(3) For each constraint table in turn, identify the cell within which person falls

*(a) Constraint Table P1: Age (child/adult/pensioner) x Sex (male/female)
* [Value initially set to 0; persons not falling in table will retain this coding of 0]

COMPUTE Table_P1=0.
IF ((Age_gp=1) and (Sex=1)) Table_P1 = 1 .
IF ((Age_gp=1) and (Sex=2)) Table_P1 = 2 .
IF ((Age_gp=2) and (Sex=1)) Table_P1 = 3 .
IF ((Age_gp=2) and (Sex=2)) Table_P1 = 4 .
IF ((Age_gp=3) and (Sex=1)) Table_P1 = 5 .
IF ((Age_gp=3) and (Sex=2)) Table_P1 = 6 .

VARIABLE LABELS Table_P1 'Constraint Table P1' .
FORMAT Table_P1 (f2.0).
EXECUTE .

*(b) Constraint Table H1: HH_Size (1-4 persons) x Cars in household (0-3)
* [Value initially set to 0; households not falling in table will retain this coding of 0]

COMPUTE Table_H1=0.
IF ((HH_Size=1) and (Cars=0)) Table_H1 = 1 .
IF ((HH_Size=2) and (Cars=0)) Table_H1 = 2 .
IF ((HH_Size=3) and (Cars=0)) Table_H1 = 3 .
IF ((HH_Size=4) and (Cars=0)) Table_H1 = 4 .
IF ((HH_Size=1) and (Cars=1)) Table_H1 = 5 .
IF ((HH_Size=2) and (Cars=1)) Table_H1 = 6 .
IF ((HH_Size=3) and (Cars=1)) Table_H1 = 7 .
IF ((HH_Size=4) and (Cars=1)) Table_H1 = 8 .
IF ((HH_Size=1) and (Cars=2)) Table_H1 = 9 .
IF ((HH_Size=2) and (Cars=2)) Table_H1 = 10 .
IF ((HH_Size=3) and (Cars=2)) Table_H1 = 11 .
IF ((HH_Size=4) and (Cars=2)) Table_H1 = 12 .
IF ((HH_Size=1) and (Cars=3)) Table_H1 = 13 .
IF ((HH_Size=2) and (Cars=3)) Table_H1 = 14 .
IF ((HH_Size=3) and (Cars=3)) Table_H1 = 15 .
IF ((HH_Size=4) and (Cars=3)) Table_H1 = 16 .

VARIABLE LABELS Table_H1 'Constraint Table H1' .
FORMAT Table_H1 (f2.0).
EXECUTE .

*(4) Identify the record number of the first and last person listed in each household
*** FILE MUST BE IN ASCENDING ORDER OF (i) HH_ID and (ii) PERSONS within household BEFORE doing this***

*(a) Ensure survey sorted by (i) HH_ID; (ii) persons within household

SORT CASES BY HH_ID (A) Person (A) .

*(b) Find first/last person listed in each household

AGGREGATE
  /OUTFILE=*
  MODE=ADDVARIABLES
  /BREAK=HH_ID
  /First_Person = FIRST(Record) /Last_Person = LAST(Record).

*(5) Export selected data from SPSS into CO-formatted inputs files (one for person-level tables; one for household-level tables)
* (Two output files: one for person-level tables; one for household-level tables; results placed in folder CO_Dummy/Data)

*** FILE MUST BE IN ASCENDING ORDER OF (i) HH_ID and (ii) PERSONS within household BEFORE EXPORT ***
*** (done in (4) above); otherwise later matching of survey with CO weights file may fail                                         ***

*(a) Person-level tables first...
*[IMPORTANT: ensure that table mappings are written out in the same order used in CO-formatted table constraint input file]

WRITE OUTFILE='../CO_Dummy_constraint_mapped_survey_Persons.txt'
  TABLE
  /Table_P1 (F4.0) .
EXECUTE.

*(b) household-level tables

*(i) Delete excess cases, retaining only the first record (person) from each household
* [i.e. retaining only records where 'record' (unique person id) = 'first_person' (first person in household)]

USE ALL.
FILTER OFF.
USE ALL.
SELECT IF(Record=First_Person).
EXECUTE .

*(ii) write-out household-level mappings 
*[IMPORTANT: ensure that table mappings are written out in the same order used in CO-formatted table constraint input file,
* preceded by survey household ID and followed by First_Person, then Last_Person]

WRITE OUTFILE='../CO_Dummy_constraint_mapped_survey_Hholds.txt'
  TABLE
  /HH_ID (F8.0) Table_H1 (F4.0) First_Person (F8.0) Last_Person (F8.0) .
EXECUTE.
